@@ -55,6 +55,39 @@ except ImportError:
     from trollius import iscoroutine
     from trollius import Future
 
+try:
+    from types import AsyncGeneratorType  # python 3.5+
+except ImportError:
+    class AsyncGeneratorType(object):
+        pass
+
+
+def _create_future_of_loop(loop):
+    return loop.create_future()
+
+
+def _create_future_directly(loop=None):
+    return Future(loop=loop)
+
+
+def _create_task_of_loop(res, loop):
+    return loop.create_task(res)
+
+
+def _create_task_directly(res, loop=None):
+    return asyncio.Task(res, loop=loop)
+
+
+if sys.version_info >= (3, 4, 2):
+    _create_task = _create_task_of_loop
+    if sys.version_info >= (3, 5, 2):
+        _create_future = _create_future_of_loop
+    else:
+        _create_future = _create_future_directly
+else:
+    _create_task = _create_task_directly
+    _create_future = _create_future_directly
+
 
 config = _Config()
 
@@ -345,7 +378,7 @@ class _AsyncioApi(object):
         if result is not _unspecified and error is not _unspecified:
             raise ValueError("Cannot have both result and error.")
 
-        f = Future(loop=self._config.loop)
+        f = _create_future(loop=self._config.loop)
         if result is not _unspecified:
             resolve(f, result)
         elif error is not _unspecified:
@@ -369,7 +402,14 @@ class _AsyncioApi(object):
             if isinstance(res, Future):
                 return res
             elif iscoroutine(res):
-                return asyncio.Task(res, loop=self._config.loop)
+                return _create_task(res, loop=self._config.loop)
+            elif isinstance(res, AsyncGeneratorType):
+                raise RuntimeError(
+                    "as_future() received an async generator function; does "
+                    "'{}' use 'yield' when you meant 'await'?".format(
+                        str(fun)
+                    )
+                )
             else:
                 return create_future_success(res)
 
@@ -437,17 +477,12 @@ class _AsyncioApi(object):
         """
         callback or errback may be None, but at least one must be
         non-None.
-
-        XXX beware the "f._result" hack to get "chainable-callback" type
-        behavior.
         """
         def done(f):
             try:
                 res = f.result()
                 if callback:
-                    x = callback(res)
-                    if x is not None:
-                        f._result = x
+                    callback(res)
             except Exception:
                 if errback:
                     errback(create_failure())
@@ -478,7 +513,7 @@ class _AsyncioApi(object):
         :param delay: Time to sleep in seconds.
         :type delay: float
         """
-        return asyncio.sleep(delay)
+        return asyncio.ensure_future(asyncio.sleep(delay))
 
 
 _default_api = _AsyncioApi(config)
